/*******************************************************************
 * Fritz Fun                                                       *
 * Created by Jan-Michael Brummer                                  *
 * All parts are distributed under the terms of GPLv2. See COPYING *
 *******************************************************************/

/**
 * \file trayicon.c
 * \brief Trayicon related functions
 */

#include <ffgtk.h>
#include <menucallbacks.h>
#include <trayicon.h>
#include <osdep/osdep.h>

#ifdef HAVE_APPINDICATOR
#include <libappindicator/app-indicator.h>
AppIndicator *psIndicator;
#endif

#define MAX_LASTCALLS 5

#ifndef __MACOSX__
/** Widget containing trayicon */
static GtkStatusIcon *psTrayIcon = NULL;
#endif

/** main gtk loop */
extern GMainLoop *psLoop;
static GtkWidget *psMenu = NULL;

/**
 * \brief Create separator item with 'text'
 * \param pnText separator text
 * \return new menuitem widget
 */
GtkWidget *createSeparatorItem( gchar *pnText ) {
	GtkWidget *psLabel = NULL;
	GtkWidget *psBox = gtk_hbox_new( FALSE, 0 );
	GtkWidget *psItem = NULL;

	psItem = gtk_menu_item_new();

	psLabel = gtk_label_new( pnText );

	gtk_box_pack_start( GTK_BOX( psBox ), gtk_hseparator_new(), TRUE, TRUE, 0 );
	gtk_box_pack_start( GTK_BOX( psBox ), psLabel, FALSE, FALSE, 0 );
	gtk_box_pack_start( GTK_BOX( psBox ), gtk_hseparator_new(), TRUE, TRUE, 0 );

	g_object_set( G_OBJECT( psItem ), "child", psBox, "sensitive", FALSE, NULL );

	return psItem;
}

/**
 * \brief Create call menu item for type nType
 * \param pnNumber call number
 * \param nType call type
 * \return new menuitem widget
 */
GtkWidget *createCallItem( char *pnNumber, int nType ) {
	GtkWidget *psLabel = NULL;
	GtkWidget *psBox = gtk_vbox_new( FALSE, 0 );
	GtkWidget *psItem = NULL;
	GtkWidget *psImage = NULL;
	GdkPixbuf *psPix = NULL;
	struct sPerson *psPerson = findPersonByNumber( pnNumber );
	gchar *pnNumberText;
#ifndef __MACOSX__
	GtkWidget *psNameLabel = NULL;
	gchar *pnName;
#endif

	psItem = gtk_image_menu_item_new();

#ifdef __MACOSX__
	psLabel = gtk_label_new( "" );
	pnNumberText = g_markup_printf_escaped( "<small>%s (%s)</small>", 
			psPerson != NULL ? psPerson -> pnDisplayName : _( "unknown" ), pnNumber );
	gtk_label_set_markup( GTK_LABEL( psLabel ), pnNumberText );
	g_free( pnNumberText );

	// gtk_box_pack_start( GTK_BOX( psBox ), psNameLabel, TRUE, TRUE, 0 );
	gtk_box_pack_start( GTK_BOX( psBox ), psLabel, FALSE, FALSE, 0 );
	gtk_misc_set_alignment( GTK_MISC( psLabel ), 0.0, 0.5 );

#else	
	psLabel = gtk_label_new( "" );
	pnNumberText = g_markup_printf_escaped( "<small>%s</small>", pnNumber );
	gtk_label_set_markup( GTK_LABEL( psLabel ), pnNumberText );
	g_free( pnNumberText );

	psNameLabel = gtk_label_new( "" );
	pnName = g_markup_printf_escaped( "<b><small>%s</small></b>", psPerson != NULL ? psPerson -> pnDisplayName : _( "unknown" ) );
	gtk_label_set_markup( GTK_LABEL( psNameLabel ), pnName );
	g_free( pnName );

	gtk_box_pack_start( GTK_BOX( psBox ), psNameLabel, TRUE, TRUE, 0 );
	gtk_box_pack_start( GTK_BOX( psBox ), psLabel, FALSE, FALSE, 0 );
	gtk_misc_set_alignment( GTK_MISC( psNameLabel ), 0.0, 0.5 );
	gtk_misc_set_alignment( GTK_MISC( psLabel ), 0.0, 0.5 );
#endif
	g_object_set( G_OBJECT( psItem ), "child", psBox, NULL );


	psPix = getTypeIcon( nType );
	psImage = GTK_WIDGET( gtk_image_new_from_pixbuf( psPix ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psImage );
	gtk_image_menu_item_set_always_show_image( GTK_IMAGE_MENU_ITEM( psItem ), TRUE );

	return psItem;
}

/**
 * \brief Add last call to submenu - callback
 * \param psMenuWidget connect each entry to this menu
 */
void addLastcalls( GtkWidget *psMenuWidget ) {
	GList *psList = NULL;
	GtkWidget *psItem = NULL;
	GtkWidget *psSubMenu = NULL;
	int nCount = 0;

	psItem = gtk_image_menu_item_new_with_label( _( "Last calls" ) );
	psSubMenu = gtk_menu_new();
	GtkWidget *psLastCalls = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psLastCalls );
	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psItem ), psSubMenu );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );

	psList = psCallerList;
	if ( g_list_length( psList ) <= 0 ) {
		gtk_widget_set_sensitive( GTK_WIDGET( psItem ), FALSE );
		return;
	}

	psItem = createSeparatorItem( _( "Incoming" ) );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );

	psList = psCallerList;
	nCount = 0;
	while ( psList != NULL ) {
		struct sCaller *psCaller = psList -> data;

		if ( psCaller -> nType == TYPE_CALLIN ) {
			psItem = createCallItem( psCaller -> pnNumber, 1 );
			gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );
			g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconDialNumber ), psCaller -> pnNumber );
			gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );
			nCount++;

			if ( nCount == MAX_LASTCALLS ) {
				break;
			}
		}

		psList = psList -> next;
	}

	psItem = createSeparatorItem( _( "Outgoing" ) );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );

	psList = psCallerList;
	nCount = 0;
	while ( psList != NULL ) {
		struct sCaller *psCaller = psList -> data;

		if ( psCaller -> nType == TYPE_CALLOUT || psCaller -> nType == TYPE_CALLOUT_NEW ) {
			psItem = createCallItem( psCaller -> pnNumber, 3 );
			gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );
			g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconDialNumber ), psCaller -> pnNumber );
			gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );
			nCount++;

			if ( nCount == MAX_LASTCALLS ) {
				break;
			}
		}

		psList = psList -> next;
	}

	psItem = createSeparatorItem( _( "Missed" ) );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );

	psList = psCallerList;
	nCount = 0;
	while ( psList != NULL ) {
		struct sCaller *psCaller = psList -> data;

		if ( psCaller -> nType == TYPE_CALLINFAILED ) {
			psItem = createCallItem( psCaller -> pnNumber, 2 );

			gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );
			g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconDialNumber ), psCaller -> pnNumber );
			gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );
			nCount++;

			if ( nCount == MAX_LASTCALLS ) {
				break;
			}
		}

		psList = psList -> next;
	}
}

/**
 * \brief Add diversity information to submenu
 * \param psSubMenu add all information to this submenu
 */
static void addDiversity( GtkWidget *psSubMenu ) {
	struct sDiversity *psDiversity = NULL;
	GList *psList = NULL;
	GtkWidget *psItem = NULL;

	psItem = gtk_image_menu_item_new_with_label( _( "Diversity" ) );
	GtkWidget *psSubMenu3 = gtk_menu_new();
	GtkWidget *psDiversityImage = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_NETWORK, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psDiversityImage );
	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psItem ), psSubMenu3 );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );

	psList = routerGetDiversity();
	if ( psList == NULL || g_list_length( psList ) == 0 ) {
		gtk_widget_set_sensitive( psItem, FALSE );
		return;
	}

	for ( psList = routerGetDiversity(); psList != NULL && psList -> data != NULL; psList = psList -> next ) {
		psDiversity = psList -> data;

		psItem = gtk_check_menu_item_new_with_label( psDiversity -> pnName );
		gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), psDiversity -> bSet );
		gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu3 ), psItem );
		g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconSetDiversity ), psDiversity );
	}
}

static GtkWidget *createMenu( void ) {
	GtkWidget *psItem;
	GtkWidget *psMenuWidget;

	psMenuWidget = gtk_menu_new();

	/* Callmonitor */
	psItem = gtk_image_menu_item_new_with_label( _( "Call Monitor" ) );
	GtkWidget *psMonitor = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psMonitor );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconToggled ), NULL );

	/* Addressbook */
	psItem = gtk_image_menu_item_new_with_label( _( "Address Book" ) );
	GtkWidget *psAddress = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_ADD, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psAddress );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconAddressBook ), NULL );

	/* Last calls */
	addLastcalls( psMenuWidget );

	/* Functions */
	GtkWidget *psFunctions = gtk_image_menu_item_new_with_label( _( "Functions" ) );
	GtkWidget *psSubMenu1 = gtk_menu_new();
	GtkWidget *psFunctionsImage = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_EXECUTE, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psFunctions ), psFunctionsImage );

	/* Functions - Call number */
	psItem = gtk_image_menu_item_new_with_label( _( "Call number" ) );
	GtkWidget *psCallMonitor = GTK_WIDGET( gtk_image_new_from_pixbuf( getTypeIcon( 3 ) ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psCallMonitor );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu1 ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconDialNumber ), NULL );
	gtk_widget_set_sensitive( GTK_WIDGET( psItem ), routerHasDial( getActiveProfile() ) );

	/* Functions - Reconnect */
	psItem = gtk_image_menu_item_new_with_label( _( "Reconnect" ) );
	GtkWidget *psReconnect = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_REFRESH, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psReconnect );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu1 ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconReconnect ), NULL );

	/* Functions - Copy IP */
	psItem = gtk_image_menu_item_new_with_label( _( "Copy IP" ) );
	GtkWidget *psClipboard = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_COPY, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psClipboard );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu1 ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconIpToClipboard ), NULL );

	/* Functions - Diversity */
	addDiversity( psSubMenu1 );

	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psFunctions ), psSubMenu1 );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psFunctions );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );

	/* Preferences */
	psItem = gtk_image_menu_item_new_with_label( _( "Preferences" ) );
	GtkWidget *psPref = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_PREFERENCES, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psPref );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( preferences ), NULL );

	/* Profiles */
	GtkWidget *psProfilesMenu = gtk_image_menu_item_new_with_label( _( "Profiles" ) );
	GtkWidget *psSubMenu = gtk_menu_new();
	GtkWidget *psCall = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_INDEX, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psProfilesMenu ), psCall );

	/* Profiles - Manage */
	psItem = gtk_image_menu_item_new_with_label( _( "Manage" ) );
	GtkWidget *psManage = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_INDEX, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psManage );
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( ProfilesManage ), NULL );

	/* Profiles - Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );

	/* Profiles - Profile list */
	GList *psList = getProfiles();
	struct sProfile *psProfile = NULL;
	psItem = NULL;
	GSList *psGroup = NULL;

	while ( psList != NULL ) {
		psProfile = psList -> data;
		if ( psProfile != NULL ) {
			psItem = gtk_radio_menu_item_new_with_label( psGroup, psProfile -> pnName );
			psGroup = gtk_radio_menu_item_get_group( GTK_RADIO_MENU_ITEM( psItem ) );
			if ( psProfile == getActiveProfile() ) {
				gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), TRUE );
			}
			gtk_menu_shell_append( GTK_MENU_SHELL( psSubMenu ), psItem );
			g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconProfile ), psProfile -> pnName );
		}
		psList = psList -> next;
	}

	gtk_menu_item_set_submenu( GTK_MENU_ITEM( psProfilesMenu ), psSubMenu );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psProfilesMenu );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );

	/* Mute */
	psItem = gtk_check_menu_item_new_with_label( _( "Mute" ) );
	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM( psItem ), callMonitorGetMute( getActiveProfile() ) );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconMute ), NULL );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );

	/* Help */
	psItem = gtk_image_menu_item_new_with_label( _( "Help" ) );
	GtkWidget *psHelp = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_HELP, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psHelp );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( trayIconHelp ), NULL );

	/* About */
	psItem = gtk_image_menu_item_new_with_label( _( "About" ) );
	GtkWidget *psAbout = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psAbout );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( aboutDialog ), NULL );

	/* Line */
	psItem = gtk_separator_menu_item_new();
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
#ifndef __MACOSX__
	/* Quit */
	psItem = gtk_image_menu_item_new_with_label( _( "Quit" ) );
	GtkWidget *psQuit = GTK_WIDGET( gtk_image_new_from_stock( GTK_STOCK_QUIT, GTK_ICON_SIZE_MENU ) );
	gtk_image_menu_item_set_image( GTK_IMAGE_MENU_ITEM( psItem ), psQuit );
	gtk_menu_shell_append( GTK_MENU_SHELL( psMenuWidget ), psItem );
	g_signal_connect( G_OBJECT( psItem ), "activate", G_CALLBACK( destroy ), NULL );
#endif

	return psMenuWidget;
}

#ifdef G_OS_WIN32
/* This is a workaround for a bug in windows GTK+. Clicking outside of the
 * menu does not get rid of it, so instead we get rid of it as soon as the
 * pointer leaves the menu.
 */

static gboolean hide_docklet_menu( gpointer data ) {
	if (data != NULL) {
		gtk_menu_popdown( GTK_MENU( data ) );
	}

	return FALSE;
}

gboolean trayIconLeaveEnter( GtkWidget *psMenuWidget, GdkEventCrossing *psEvent, void *pData ) {
	static guint nHideTimer = 0;

	if ( psEvent -> type == GDK_LEAVE_NOTIFY && ( psEvent -> detail == GDK_NOTIFY_ANCESTOR || psEvent -> detail == GDK_NOTIFY_UNKNOWN ) ) {
		if ( nHideTimer == 0 ) {
			nHideTimer = g_timeout_add( 500, hide_docklet_menu, psMenuWidget );
		}
	} else if ( psEvent -> type == GDK_ENTER_NOTIFY && psEvent -> detail == GDK_NOTIFY_ANCESTOR ) {
		if ( nHideTimer != 0 ) {
			g_source_remove( nHideTimer );
			nHideTimer = 0;
		}
	}

	return FALSE;
}
#endif

#ifndef __MACOSX__
/**
 * \brief Handle tray icon menu
 * \param psStatusIcon status icon pointer
 * \param nButton which button
 * \param nActivationTime activation time
 * \param pUserData user data pointer
 */
static void trayIconOnMenu( GtkStatusIcon *psStatusIcon, guint nButton, guint nActivateTime, gpointer pUserData) {
	if ( psMenu != NULL ) {
		gtk_widget_destroy( psMenu );
		psMenu = NULL;
	}
	psMenu = createMenu();

#ifdef G_OS_WIN32
	/* Workaround for several Windows bugs */
	g_signal_connect( psMenu, "leave-notify-event", G_CALLBACK( trayIconLeaveEnter ), NULL );
	g_signal_connect( psMenu, "enter-notify-event", G_CALLBACK( trayIconLeaveEnter ), NULL );
#endif /* G_OS_WIN32 */
	gtk_widget_show_all( psMenu );

#ifdef G_OS_WIN32
	gtk_menu_popup( GTK_MENU( psMenu ), NULL, NULL, gtk_status_icon_position_menu, psStatusIcon, 0, nActivateTime );
#else
	gtk_menu_popup( GTK_MENU( psMenu ), NULL, NULL, gtk_status_icon_position_menu, psStatusIcon, nButton, nActivateTime );
#endif /* G_OS_WIN32 */
}

/**
 * \brief Create tray icon and set menu/tooltip
 */
void createTrayIcon( void ) {
	/* TODO set Mac tooltips */
	psTrayIcon = gtk_status_icon_new();
	g_signal_connect( G_OBJECT( psTrayIcon ), "popup-menu", G_CALLBACK( trayIconOnMenu ), NULL );
	g_signal_connect( G_OBJECT( psTrayIcon ), "activate", G_CALLBACK( trayIconToggled ), NULL );
	trayIconSetIconType( TYPE_CALLOUT );
	gtk_status_icon_set_tooltip_text( psTrayIcon,  "ffgtk" );
	gtk_status_icon_set_visible( psTrayIcon, TRUE );
}
#endif /* __MACOSX__ */

/**
 * \brief Periodically update tooltip information
 * \param pUserData pointer to tray icon
 * \return TRUE
 */
gboolean updateTooltip( gpointer pUserData ) {
	gchar *pnIp = NULL;
	gchar *pnText = NULL;
	gint nMaxUp = 0;
	gint nMaxDown = 0;

	pnIp = routerGetIp( getActiveProfile() );

	if ( pnIp != NULL ) {
		routerGetSpeed( getActiveProfile(), &nMaxUp, &nMaxDown );
		pnText = g_strdup_printf( _( "IP: %s\nMax. Up: %d kbit/s\nMax. Down: %d kbit/s" ), pnIp, nMaxUp >> 10, nMaxDown >> 10 );
		g_free( pnIp );
		pnIp = NULL;
	} else {
		pnText = g_strdup_printf( _( "Not connected" ) );
	}

	GtkStatusIcon *psIcon = ( GtkStatusIcon * ) pUserData;
	gtk_status_icon_set_tooltip_text( psIcon, pnText );

	g_free( pnText );
	pnText = NULL;

	return TRUE;
}

/**
 * \brief Get tray icon widget
 * \return trayicon widget
 */
GtkStatusIcon *getTrayIcon( void ) {
#ifndef __MACOSX__
	return psTrayIcon;
#else
	return NULL;
#endif 
}

/**
 * \brief Set which tray icon to use 
 * \input type if icon    
 */

void trayIconSetIconType( int nType ) {
	Debug( KERN_DEBUG, "Trayicon type set to %d\n", nType );
#ifdef __MACOSX__
	GtkosxApplication *psTheApp = g_object_new(GTKOSX_TYPE_APPLICATION, NULL);
	gtkosx_application_set_dock_icon_pixbuf( psTheApp, getTypeIcon( nType ) );
#elif defined ( HAVE_APPINDICATOR )
	if ( nType == TYPE_CALLINFAILED ) {
		app_indicator_set_icon( psIndicator, "callinfailed" );
	} else {
		app_indicator_set_icon( psIndicator, "ffgtk" );
	}
#else
	gtk_status_icon_set_from_pixbuf( GTK_STATUS_ICON( getTrayIcon() ), getTypeIcon( nType ) );
#endif
}

#if defined ( HAVE_APPINDICATOR ) || defined ( __MACOSX__ )
/** menu update id */
static gint nMenuUpdateId = 0;

/**
 * \brief Update menu signal callback
 * \return FALSE
 */
static gboolean updateMenu( void ) {
	GtkWidget *psMenuNew;

	psMenuNew = createMenu();
	gtk_widget_show_all( GTK_WIDGET( psMenuNew ) );

#ifdef HAVE_APPINDICATOR
	app_indicator_set_menu( psIndicator, GTK_MENU( psMenuNew ) );
#elif defined ( __MACOSX__)
	GtkosxApplication *psTheApp = g_object_new( GTKOSX_TYPE_APPLICATION, NULL);
	gtkosx_application_set_dock_menu( psTheApp, GTK_MENU_SHELL( psMenuNew ) );
#endif

	nMenuUpdateId = 0;
	if ( psMenu != NULL ) {
		gtk_widget_destroy( psMenu );
	}
	psMenu = psMenuNew;

	return FALSE;
}
#endif /* HAVE_APPINDICATOR  || __MACOSX__*/

/**
 * \brief Recreate app menu
 */
void trayIconRecreateAppMenu( void ) {
#ifdef HAVE_APPINDICATOR
	if ( !nMenuUpdateId ) {
		nMenuUpdateId = g_idle_add_full( G_PRIORITY_LOW, ( GSourceFunc ) updateMenu, NULL, NULL );
	}
#elif defined ( __MACOSX__ )
	/* gtkosxintegration does not like it when we change the menu once it is set */
	/* the dock menu does not have a sync function */
	if ( !nMenuUpdateId && !psMenu ) {
		nMenuUpdateId = g_idle_add_full( G_PRIORITY_LOW, ( GSourceFunc ) updateMenu, NULL, NULL );
	}
#endif /*  __MACOSX__ */
}

#ifdef __MACOSX__
/**
 * \brief Open call monitor only if no other windows are active
 */
void dockMenuOpenCallMonitor( void ) {
	/* we need to check if any window is open as NSApplicationDidBecomeActive */
	/* iget fired if any other open window is selected */
	if (getOpenWindows() == 0) {
		CallMonitor();
	} else {
		Debug( KERN_DEBUG, "Activated while window %d is  open\n", getOpenWindows() );
	}
}
#endif

/**
 * \brief Initialize tray icon structure and start update tooltip thread
 */
void trayIconInit( void ) {
#ifdef __MACOSX__
	GtkosxApplication *psTheApp = g_object_new( GTKOSX_TYPE_APPLICATION, NULL );

	trayIconSetIconType( TYPE_CALLOUT );

	g_signal_connect( psTheApp, "NSApplicationDidBecomeActive", G_CALLBACK( dockMenuOpenCallMonitor ), NULL );
	g_signal_connect( psTheApp, "NSApplicationBlockTermination", G_CALLBACK( destroy ), NULL );
	g_signal_connect( psTheApp, "NSApplicationWillTerminate", G_CALLBACK( destroy ), NULL );

	/* the menu itself will get created on the first call to trayIconRecreateAppMenu() */
#elif defined ( HAVE_APPINDICATOR )
	gchar *pnDir = getPkgDataDir();

	psIndicator = app_indicator_new_with_path( "ffgtk", "ffgtk", APP_INDICATOR_CATEGORY_APPLICATION_STATUS, pnDir );
	app_indicator_set_status( psIndicator, APP_INDICATOR_STATUS_ACTIVE );

	trayIconRecreateAppMenu();
#else /* neither MACOSX nor AppIndicator */
	createTrayIcon();
	g_timeout_add_seconds( 15, updateTooltip, psTrayIcon );
#endif
}
